//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.unisi.ch/carzaniga/siena/
//
//  Author: Antonio Carzaniga (firstname.lastname@unisi.ch)
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 1998-2002 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to Antonio Carzaniga (firstname.lastname@unisi.ch).
//
package sienatest;

import siena.*;
import siena.comm.*;

public class TestSenderPerformance {
    static PacketSenderFactory sender_factory = new GenericSenderFactory();

    public static void main(String argv[]) {
	if (argv.length != 3) {
	    System.out.println("usage: TestSenderPerformance <address> <packet-count> <packet-length>");
	    System.exit(1);
	}
	try {
	    PacketSender sender = sender_factory.createPacketSender(argv[0]);

	    int count = Integer.decode(argv[1]).intValue();
	    int size =  Integer.decode(argv[2]).intValue();

	    byte[] buf = new byte[size];

	    long start = System.currentTimeMillis();
	    
	    for(int i = 0; i < count; ++i) 
		sender.send(buf);

	    sender.send(new byte[1]);
	    
	    long elapsed = System.currentTimeMillis() - start;

	    System.out.println("ms=" + elapsed + " count=" + count);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
